@@ -14,6 +14,7 @@
 #ifndef _MPG123_AUDIO_H_
 #define _MPG123_AUDIO_H_
 
+#include "compat.h"
 #include "mpg123.h"
 #include "module.h"
 
@@ -92,5 +93,14 @@ int reset_output(audio_output_t *ao);
 void output_pause(audio_output_t *ao);  /* Prepare output for inactivity. */
 void output_unpause(audio_output_t *ao); /* Reactivate output (buffer process). */
 
+void audio_enclist(char** list); /* Make a string of encoding names. */
+
+/*
+	Twiddle audio output rate to yield speedup/down (pitch) effect.
+	The actually achieved pitch value is stored in param.pitch.
+	Returns 1 if pitch setting succeeded, 0 otherwise.
+*/
+int set_pitch(mpg123_handle *fr, audio_output_t *ao, double new_pitch);
+
 #endif
 
