@@ -1,7 +1,7 @@
 /*
 	buffer.c: output buffer
 
-	copyright 1997-2006 by the mpg123 project - free software under the terms of the LGPL 2.1
+	copyright 1997-2009 by the mpg123 project - free software under the terms of the LGPL 2.1
 	see COPYING and AUTHORS files in distribution or http://mpg123.org
 	initially written by Oliver Fromme
 
@@ -16,6 +16,7 @@
 
 #include "common.h"
 #include <errno.h>
+#include "debug.h"
 
 int outburst = 32768;
 
@@ -92,15 +93,17 @@ extern int buffer_pid;
 
 void buffer_sig(int signal, int block)
 {
-	if (!buffermem)
-		return;
+	if (!buffermem) return;
 
-	kill(buffer_pid, signal);
-	
 	if (!block)
+	{ /* Just signal, do not wait for anything. */
+		kill(buffer_pid, signal);
 		return;
+	}
 
-	if(xfermem_block(XF_WRITER, buffermem) != XF_CMD_WAKEUP) 
+	/* kill() and the waiting needs to be taken care of properly for parallel execution.
+	  Nobody reported issues so far, but I want to be sure. */
+	if(xfermem_sigblock(XF_WRITER, buffermem, buffer_pid, signal) != XF_CMD_WAKEUP) 
 		perror("Could not resync/reset buffers");
 	return;
 }
@@ -137,6 +140,7 @@ void buffer_loop(audio_output_t *ao, sigset_t *oldsigset)
 		else if(cmd == XF_CMD_WAKEUP)
 		{
 			debug("got wakeup... leaving config mode");
+			xfermem_putcmd(buffermem->fd[XF_READER], XF_CMD_WAKEUP);
 			break;
 		}
 		else
@@ -149,13 +153,15 @@ void buffer_loop(audio_output_t *ao, sigset_t *oldsigset)
 	/* Fill complete buffer on first run before starting to play.
 	 * Live mp3 streams constantly approach buffer underrun otherwise. [dk]
 	 */
-	preload = xf->size;
+	preload = (int)(param.preload*xf->size);
+	if(preload > xf->size) preload = xf->size;
+	if(preload < 0) preload = 0;
 
 	for (;;) {
 		if (intflag) {
 			debug("handle intflag... flushing");
 			intflag = FALSE;
-			if (param.outmode == DECODE_AUDIO) ao->flush(ao);
+			ao->flush(ao);
 			/* Either prepare for waiting or empty buffer now. */
 			if(!xf->justwait) xf->readindex = xf->freeindex;
 			else
