@@ -1,7 +1,7 @@
 /*
 	httpget.c: http communication
 
-	copyright ?-2008 by the mpg123 project - free software under the terms of the LGPL 2.1
+	copyright ?-2011 by the mpg123 project - free software under the terms of the LGPL 2.1
 	see COPYING and AUTHORS files in distribution or http://mpg123.org
 	initially written Oliver Fromme
 	old timestamp: Wed Apr  9 20:57:47 MET DST 1997
@@ -32,7 +32,6 @@
 #ifdef NETWORK
 #include "resolver.h"
 
-#define HTTP_MAX_RELOCATIONS 20
 #include <errno.h>
 #include "true.h"
 #endif
@@ -76,7 +75,9 @@ static const char* mime_file[] =
 	"audio/mp3",   "audio/x-mp3", 
 	"audio/mpeg3", "audio/x-mpeg3",
 	"audio/mpg",   "audio/x-mpg",
-	"audio/x-mpegaudio", NULL
+	"audio/x-mpegaudio",
+	"application/octet-stream", /* Assume raw binary data is some MPEG data. */
+	NULL
 };
 static const char* mime_m3u[] = { "audio/mpegurl", "audio/mpeg-url", "audio/x-mpegurl", NULL };
 static const char* mime_pls[]	= { "audio/x-scpls", "audio/scpls", "application/pls", NULL };
@@ -119,8 +120,8 @@ debunk_result:
 
 
 #ifdef NETWORK
-
-int writestring (int fd, mpg123_string *string)
+#if !defined (WANT_WIN32_SOCKETS)
+static int writestring (int fd, mpg123_string *string)
 {
 	size_t result, bytes;
 	char *ptr = string->p;
@@ -128,7 +129,8 @@ int writestring (int fd, mpg123_string *string)
 
 	while(bytes)
 	{
-		if((result = write(fd, ptr, bytes)) < 0 && errno != EINTR)
+		result = write(fd, ptr, bytes);
+		if(result < 0 && errno != EINTR)
 		{
 			perror ("writing http string");
 			return FALSE;
@@ -144,8 +146,10 @@ int writestring (int fd, mpg123_string *string)
 	return TRUE;
 }
 
-size_t readstring (mpg123_string *string, size_t maxlen, FILE *f)
+static size_t readstring (mpg123_string *string, size_t maxlen, FILE *f)
 {
+	int err;
+	debug2("Attempting readstring on %d for %"SIZE_P" bytes", f ? fileno(f) : 0, (size_p)maxlen);
 	string->fill = 0;
 	while(maxlen == 0 || string->fill < maxlen)
 	{
@@ -156,8 +160,9 @@ size_t readstring (mpg123_string *string, size_t maxlen, FILE *f)
 			string->fill = 0;
 			return 0;
 		}
+		err = read(fileno(f),string->p+string->fill,1);
 		/* Whoa... reading one byte at a time... one could ensure the line break in another way, but more work. */
-		if( read(fileno(f),string->p+string->fill,1) == 1)
+		if( err == 1)
 		{
 			string->fill++;
 			if(string->p[string->fill-1] == '\n') break;
@@ -182,6 +187,7 @@ size_t readstring (mpg123_string *string, size_t maxlen, FILE *f)
 	}
 	return string->fill;
 }
+#endif /* WANT_WIN32_SOCKETS */
 
 void encode64 (char *source,char *destination)
 {
@@ -246,17 +252,11 @@ void get_header_string(mpg123_string *response, const char *fieldname, mpg123_st
 	}
 }
 
-/* needed for HTTP/1.1 non-pipelining mode */
-/* #define CONN_HEAD "Connection: close\r\n" */
-#define CONN_HEAD ""
-
 /* shoutcsast meta data: 1=on, 0=off */
-static const char *icy_yes = "Icy-MetaData: 1\r\n";
-static const char *icy_no  = "Icy-MetaData: 0\r\n";
 
 char *httpauth = NULL;
 
-static size_t accept_length(void)
+size_t accept_length(void)
 {
 	int i,j;
 	static size_t l = 0;
@@ -270,7 +270,7 @@ static size_t accept_length(void)
 }
 
 /* Returns TRUE or FALSE for success. */
-static int proxy_init(struct httpdata *hd)
+int proxy_init(struct httpdata *hd)
 {
 	int ret = TRUE;
 	/* If we don't have explicit proxy given, probe the environment. */
@@ -331,7 +331,7 @@ static int append_accept(mpg123_string *s)
 	What about converting them to "+" instead? Would make things a lot easier.
 	Or, on the other hand... what about avoiding HTML encoding at all?
 */
-static int translate_url(const char *url, mpg123_string *purl)
+int translate_url(const char *url, mpg123_string *purl)
 {
 	const char *sptr;
 	/* The length of purl is upper bound by 3*strlen(url) + 1 if
@@ -372,7 +372,7 @@ static int translate_url(const char *url, mpg123_string *purl)
 	return TRUE;
 }
 
-static int fill_request(mpg123_string *request, mpg123_string *host, mpg123_string *port, mpg123_string *httpauth1, int *try_without_port)
+int fill_request(mpg123_string *request, mpg123_string *host, mpg123_string *port, mpg123_string *httpauth1, int *try_without_port)
 {
 	char* ttemp;
 	int ret = TRUE;
@@ -447,7 +447,7 @@ static int fill_request(mpg123_string *request, mpg123_string *host, mpg123_stri
 
 	return ret;
 }
-
+#if !defined (WANT_WIN32_SOCKETS)
 static int resolve_redirect(mpg123_string *response, mpg123_string *request_url, mpg123_string *purl)
 {
 	debug1("request_url:%s", request_url->p);
@@ -497,7 +497,7 @@ int http_open(char* url, struct httpdata *hd)
 	int oom  = 0;
 	int relocate, numrelocs = 0;
 	int got_location = FALSE;
-	FILE *myfile;
+	FILE *myfile = NULL;
 	/*
 		workaround for http://www.global24music.com/rautemusik/files/extreme/isdn.pls
 		this site's apache gives me a relocation to the same place when I give the port in Host request field
@@ -574,13 +574,13 @@ int http_open(char* url, struct httpdata *hd)
 		if(!fill_request(&request, &host, &port, &httpauth1, &try_without_port)){ oom=1; goto exit; }
 
 		httpauth1.fill = 0; /* We use the auth data from the URL only once. */
-
-		if((sock = open_connection(&host, &port)) < 0)
+		debug2("attempting to open_connection to %s:%s", host.p, port.p);
+		sock = open_connection(&host, &port);
+		if(sock < 0)
 		{
 			error1("Unable to establish connection to %s", host.fill ? host.p : "");
 			goto exit;
 		}
-
 #define http_failure close(sock); sock=-1; goto exit;
 		
 		if(param.verbose > 2) fprintf(stderr, "HTTP request:\n%s\n",request.p);
@@ -685,6 +685,7 @@ exit: /* The end as well as the exception handling point... */
 	mpg123_free_string(&httpauth1);
 	return sock;
 }
+#endif /*WANT_WIN32_SOCKETS*/
 
 #else /* NETWORK */
 
